local addonName, addonTab = ... --[[	addonName = The name of this addon as in the TOC file and folder name. addonTab = The shared addon table between the Lua files of this addon.]]
local L = addonTab.L -- Localization
local DejaMarkFrame -- The main frame local reference.
addonTab.version = GetAddOnMetadata(addonName, "Version")
addonTab.addoninfo = 'v'..addonTab.version
addonTab.WoWClient = {
	Retail = (WOW_PROJECT_ID == WOW_PROJECT_MAINLINE),
	Wrath = (WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC),
	TBCC = (WOW_PROJECT_ID == WOW_PROJECT_BURNING_CRUSADE_CLASSIC),
	Vanilla = (WOW_PROJECT_ID == WOW_PROJECT_CLASSIC),
}

BINDING_HEADER_DEJAMARK = "DejaMark"
_G["BINDING_NAME_CLICK DejaMark:LeftButton"] = "Show Markers(Hold)"

DM_TARGET_MARKER_ORDER = {};
DM_TARGET_MARKER_ORDER[1] = { name = "Star", wm = 5, wmid = 85000 }		-- Star = TM1, WF5
DM_TARGET_MARKER_ORDER[2] = { name = "Circle", wm = 6, wmid = 171555 }	-- Circle = TM2, WF6
DM_TARGET_MARKER_ORDER[3] = { name = "Diamond", wm = 3, wmid = 84998 }	-- Diamond = TM3, WF3
DM_TARGET_MARKER_ORDER[4] = { name = "Triangle", wm = 2, wmid = 84997 }	-- Triangle = TM4, WF2
DM_TARGET_MARKER_ORDER[5] = { name = "Moon", wm = 7, wmid = 171556 }	-- Moon = TM5, WF7
DM_TARGET_MARKER_ORDER[6] = { name = "Square", wm = 1, wmid = 84996 }	-- Square = TM6, WF1
DM_TARGET_MARKER_ORDER[7] = { name = "Cross", wm = 4, wmid = 84999 }	-- Cross = TM7, WF4
DM_TARGET_MARKER_ORDER[8] = { name = "Skull", wm = 8, wmid = 171557 }	-- Skull = TM8, WF8
DM_TARGET_MARKER_ORDER[9] = { name = "None", wm = 9, wmid = nil }		-- None = TM9, WF0

function DM_ClearPlayer()
	local playerindex = GetRaidTargetIndex("player")
	if playerindex then
		SetRaidTargetIcon("player", playerindex)
	else
		SetRaidTarget("player", 0)
	end
end

function DM_DesatMarks()
	if ( addonTab.WoWClient.Retail ) then
		for i = 1, 9 do -- Including 9 to popup on mouseover even tho it will only ever be chaked when in ColorPicker mode.
			local desatButton = _G["DejaMarkIconButton"..i]
			local checked = addonTab.db.DejaMark.maskcolorchecked or IsRaidMarkerActive(DM_TARGET_MARKER_ORDER[i].wm)
			desatButton.Texture:SetAllPoints(desatButton);
			if ( not checked ) then
				desatButton:SetScale(1.0)
				desatButton.Texture:SetDesaturated(false)
				desatButton.Overlay:SetColorTexture(0, 0, 1, 0)
			end
			if ( checked ) or ( ( desatButton:IsMouseOver() ) and ( checked ) ) then
				-- local maskcolor = addonTab.db.DejaMark.maskcolor
				local r, g, b, a = unpack(addonTab.db.DejaMark.maskcolor)
				desatButton:SetScale(0.84)
				desatButton.Texture:SetDesaturated(true)
				desatButton.Overlay:SetColorTexture(r, g, b, a)
			end
			if ( desatButton:IsMouseOver() ) and ( not checked ) then
				desatButton.Texture:ClearAllPoints();
				desatButton.Texture:SetPoint("TOPLEFT", -5, 5);
				desatButton.Texture:SetPoint("BOTTOMRIGHT", 5, -5);
			end
		end
	end
end

local function DM_UpdateMarks()
	C_Timer.After(0.16, DM_DesatMarks) --[[ The server takes 0.150 seconds to process the batch on its side. Adding 0.01 to prevent sync/alignment truncation. ]]
end

--[[ Create and set the main frame elements ]]
addonTab.mainFrame = CreateFrame("Frame", addonName.."Frame", UIParent)
DejaMarkFrame = addonTab.mainFrame --[[ Alias and localize the addon namespace table value from the object reference; i.e. make the name descriptive and obvious that it is the object and not a table value. ]]

	--[[ Register main frame events ]]
	DejaMarkFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	DejaMarkFrame:RegisterEvent("EXECUTE_CHAT_LINE")

	--[[ Set Size, Scale, Points, and Interactions ]]
	DejaMarkFrame:SetClampedToScreen( true )
	DejaMarkFrame:SetMovable(true)
	DejaMarkFrame:EnableMouse(true)
	DejaMarkFrame:SetSize(100,100)
	DejaMarkFrame:SetScale(1.0)
	DejaMarkFrame:Hide()

	--[[ Set Textures and Backdrop ]]
	local t=DejaMarkFrame:CreateTexture(nil,"ARTWORK")
		t:SetAllPoints(DejaMarkFrame)
		t:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Background")

--[[ Set Scripts ]]
DejaMarkFrame:SetScript("OnShow", function(self)
	DM_DesatMarks()
end)

DejaMarkFrame:SetScript("OnEvent", function(self, event, arg1, arg2, arg3, arg4, ...)
	if ( event == "EXECUTE_CHAT_LINE" ) then
		for key, wmid in pairs(DM_TARGET_MARKER_ORDER) do
			if ( "/wm "..DM_TARGET_MARKER_ORDER[key].wm == arg1 ) or ( "/cwm "..DM_TARGET_MARKER_ORDER[key].wm == arg1 ) then
				C_Timer.After(0.27, DM_DesatMarks) --[[ The server takes 0.25 seconds to process/throttle chat commands, presumably in a batch. Adding 0.02 to prevent sync/alignment truncation. ]]
			end
		end
		if ( arg1 == "/run ClearRaidMarker()" )  then
			C_Timer.After(0.27, DM_DesatMarks) --[[ The server takes 0.25 seconds to process/throttle chat commands, presumably in a batch. Adding 0.02 to prevent sync/alignment truncation. ]]
		end
	end
	if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		if ( arg1 == "player" ) then
			for key, wmid in pairs(DM_TARGET_MARKER_ORDER) do
				if DM_TARGET_MARKER_ORDER[key].wmid == arg3 then
					DM_UpdateMarks()
					-- World Marker Flares' Cooldowns Debugging
					-- local cooldownMS, gcdMS = GetSpellBaseCooldown(DM_TARGET_MARKER_ORDER[key].wmid) --World Marker Flares' Debugging Cooldowns
					-- print(DM_TARGET_MARKER_ORDER[key].name..":|nCooldown: "..format("%.2f", cooldownMS/1000).."s|nGCD: "..format("%.2f", gcdMS/1000).."s") --World Marker Flares' Debugging Cooldowns
				end
			end
		end
	end
end)

for key, v in ipairs(DM_TARGET_MARKER_ORDER) do
	local wflare = DM_TARGET_MARKER_ORDER[key].wm
	Button = CreateFrame("Button", "DejaMarkIconButton"..key, DejaMarkFrame, "SecureActionButtonTemplate"); --Use "SecureActionButtonTemplate" if using macros
	Button:RegisterForClicks("LeftButtonDown", "LeftButtonUp", "RightButtonDown", "RightButtonUp")
	Button:SetWidth(32);
	Button:SetHeight(32);
	if(key==9)then
		Button:SetPoint("BOTTOMRIGHT");
	elseif(key==1)then
		Button:SetPoint("TOPLEFT");
	elseif(key==2)then
		Button:SetPoint("TOP");
	elseif(key==3)then
		Button:SetPoint("TOPRIGHT");
	elseif(key==4)then
		Button:SetPoint("LEFT");
	elseif(key==5)then
		Button:SetPoint("BOTTOM");
	elseif(key==6)then
		Button:SetPoint("RIGHT");
	elseif(key==7)then
		Button:SetPoint("BOTTOMLEFT");
	elseif(key==8)then
		Button:SetPoint("CENTER");
	end

	Button.Texture = Button:CreateTexture(Button:GetName().."NormalTexture", "ARTWORK");
	Button.Overlay = Button:CreateTexture(Button:GetName().."OverlayTexture", "ARTWORK");
	Button.Mask = Button:CreateMaskTexture();

	if(key==9)then
		Button.Texture:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up");
		Button.Mask:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up");
	else
		Button.Texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_"..key);
		Button.Mask:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_"..key);
		-- SetRaidTargetIconTexture(Button.Texture, key); -- Blizzard specific function that centers the sprite texture via index but doesn't include clearing all marks.
		-- SetRaidTargetIconTexture(Button.Mask, key); -- Blizzard specific function that centers the sprite texture via index but doesn't include clearing all marks.
	end

	Button.Texture:SetAllPoints(Button);
	Button.Overlay:SetAllPoints(Button);
	Button.Mask:SetAllPoints(Button.Overlay);
	Button.Overlay:AddMaskTexture(Button.Mask);

	Button:SetScript("OnEnter", function(self, ...)
		DM_UpdateMarks()
	end);

	Button:SetScript("OnLeave", function(self, ...)
		DM_UpdateMarks()
	end);

	--[[ Target Markers ]]--
	Button:HookScript("OnClick", function (self, button, down) -- We have to hook because attributes below are secure/restricted.
		if (not down) and (button=="LeftButton") then
			if (key == 9) then
				for i=8,0,-1 do
					SetRaidTarget("player", i) -- SetRaidTarget is a hard set.  It will not toggle the icon on/off
				end
				C_Timer.NewTicker(0.16, DM_ClearPlayer, 4)
			else
				SetRaidTargetIcon("target", key) -- SetRaidTargetIcon toggles the icon on/off
			end
		end
		DM_UpdateMarks()
	end)

	--[[ World Marker Flares ]]--
	if ( addonTab.WoWClient.Retail ) then
		if(key==9)then
			Button:SetAttribute("*type2", "macro");
			Button:SetAttribute("*macrotext2", '/run ClearRaidMarker()');
		else
			Button:SetAttribute("*type2", "macro");
			Button:SetAttribute("*macrotext2", '/wm '..DM_TARGET_MARKER_ORDER[key].wm..'\n/cwm '..DM_TARGET_MARKER_ORDER[key].wm);
		end
	end
	
	--Create Cooldown Animation Frames
	Button.cooldown = CreateFrame("Cooldown", "CooldownAnimation"..DM_TARGET_MARKER_ORDER[key].wm, Button, "CooldownFrameTemplate")
	Button.cooldown:SetSwipeTexture("Interface\\ICONS\\Ability_Ambush")
	Button.cooldown:SetEdgeTexture("Interface\\AddOns\\DejaAuras\\Textures\\SetEdgeTexture_Clear");
	Button.cooldown:SetHideCountdownNumbers(false); -- Hides the countdown numbers regardless of the cvar setting
	
	--[[ 
		Cooldown Animations are a design choice that give a visual indication 
		of when each marker's cooldown has ended and button can be clicked again 
		to place on the cursor and in the field. Clicking before the cooldown has 
		ended does still allow for the marker to be immediatly cleared. 
	]]--

	--Create Cooldown Textures
	Button.cooldown.texture = Button.cooldown:CreateTexture(nil,"BACKGROUND")
	Button.cooldown.texture:SetAllPoints("CooldownAnimation"..DM_TARGET_MARKER_ORDER[key].wm)
	Button.cooldown.texture:SetMask("Interface\\ICONS\\Ability_Ambush")
	if(key==9)then
		Button.Texture:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up");
	else
		Button.cooldown.texture:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcon_"..key)
	end
end

-- DejaMark base toggle button has no visible parts. It exists as a place to accept a click to run a snippet
addonTab.toggleframe = CreateFrame("Button", "DejaMark", UIParent, "SecureHandlerClickTemplate")
local toggleframe = addonTab.toggleframe
	toggleframe:RegisterForClicks("AnyUp", "AnyDown")
	toggleframe:SetFrameRef("ParentFrame", DejaMarkFrame)
	
	toggleframe:SetAttribute("_onclick", [[
		ParentFrame = self:GetFrameRef("ParentFrame")
		if down then
			ParentFrame:SetPoint("CENTER","$cursor")
			ParentFrame:Show()
		else
			ParentFrame:Hide()
		end
	]])
